/*
 * This file is part of SpongeAPI, licensed under the MIT License (MIT).
 *
 * Copyright (c) SpongePowered <https://www.spongepowered.org>
 * Copyright (c) contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.spongepowered.api.event.impl;

import org.spongepowered.api.event.Event;
import org.spongepowered.api.eventgencore.annotation.SetField;
import org.spongepowered.api.util.event.callback.CallbackList;

/**
 * An abstract event that can be extended for any and all custom events as
 * necessary.
 */
public abstract class AbstractEvent implements Event {

    @SetField
    protected final CallbackList callbacks = new CallbackList();

    @Override
    public CallbackList getCallbacks() {
        return this.callbacks;
    }

    /**
     * Called once all fields have been set by the generated
     * constructor in a subclass.
     *
     * <p>This method should be used
     * to initialize any fields that depend on parameters
     * passed to the constructor.</p>
     */
    protected void init() {}

}
